#pragma rtGlobals=1		// Use modern global access method.
#pragma hide=1

//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surace and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////



menu "XPST"
	submenu "Background subtraction"
		"Point-Baseline",LaunchLinePanel()
	end 	
end


macro LaunchLinePanel()
	String /G root:LP_workingFolder
	DoWindow linePanel     //will write 1 into V_Flag, if the window already exists
	if ( V_Flag )
		DoWindow /F linePanel   //bring it to front
		return -1                          //exit and do nothing else
	endif
	root:LP_workingFolder = GetDataFolder(1)
	linePanel()
end
//define the gui and the global variables

Window linePanel() : Panel
	initializeLine()
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=2 /W=(201,62,955,683) as "Baseline"
	//SetDrawLayer UserBack
	DrawText 21,30,"Please select a wave"
	DrawText 21,118,"Move the Cursor and press 'Add'"
	DrawText 21,137,"At least 2 points required!"
	GroupBox PopUpGroup,pos={10,5},size={270,80}
	PopupMenu WavePop,pos={26,40},size={240,21},bodyWidth=240,proc=initLinePopUp
	PopupMenu WavePop,mode=1,popvalue="_select_",value= #"WaveList(\"!*Disp*\",\";\",\"\")"
	Button okBut,pos={274,111},size={115,25},disable = 2,proc=printMe,title="Refresh"
	Button CancBut,pos={285,7},size={115,25},proc=LeaveLine,title="Undo and Exit"
	Button ExitBut,pos={286,37},size={114,26},proc=exitLineWindow,title="Accept and Exit"
	GroupBox GraphGroup,pos={11,100},size={390,500}
	Button AddBut,pos={405,100},size={65,20},disable = 2, proc = AddCoord, title="Add >>"
	Button DelBut,pos={405,126},size={65,20},disable = 2, proc=DelCoord,title="Delete <<"
	Display/W=(17,142,395,367)/HOST=#  
	ModifyGraph rgb=(0,0,0)
	ModifyGraph mirror=2
	RenameWindow #,guiDisplay
	SetActiveSubwindow ##
	Display/W=(17,370,395,590)/HOST=#  
	ModifyGraph mirror=2
	RenameWindow #,guiDisplay2
	SetActiveSubwindow ##
	Edit /W=(476,95,729,604)/HOST=# XCoords, YCoords
	ModifyTable format(XCoords)=3, format(YCoords)=3
	RenameWindow #,CoordTable
	SetActiveSubwindow ##
EndMacro


function initializeLine()
//this function initializes the global variables (yes, I know, but Igor does not...)
//identical to the default values of the gui elements 
//avoid errors or inconsistencies
	SVAR name = root:LP_workingFolder
	SetDataFolder name
	String /G LP_nameWaveRed = StringFromList(0,WaveList("!*Disp*",";",""))
	make /d /n=0 /o XCoords, YCoords
	//string /G nameXWaveRed = StringFromList(0,WaveList("!*Disp*",";",""))
end

function initLinePopUp(ctrlName,popNum,popStr): PopupMenuControl
//as soon as a wave is selected, it is displayed and cursors appear
	string ctrlName
	variable popNum
	String popStr
	//local reference to a global variable
	SVAR value = LP_nameWaveRed
	value = popStr                              //write the selection to the global string
	wave xVals = XCoords
	wave yVals = YCoords
	
	redimension /d /n=(0)  xVals
	redimension /d /n=(0) yVals
	
	Button okBut disable=0
	Button cancBut disable=2
	Button AddBut disable = 0, fColor=(44000,52000,65500)  
	Button DelBut disable = 0, fColor=(44000,52000,65500)  
	wave WorkWave = $value

	duplicate /o /d WorkWave, LP_DispWave
	 
	RemoveFromGraph /Z /W=linePanel#guiDisplay2 /Z $"#0"  
	RemoveFromGraph /Z /W=linePanel#guiDisplay /Z $"#0" $"#1" $"#2" $"#3" $"#4"   //more traces..to be on the safe side

	AppendToGraph /w=linePanel#guiDisplay LP_DispWave
	ModifyGraph /W=linePanel#guiDisplay rgb(LP_DispWave)=(0,0,0) 
	//the name of the panel is hard-coded, usually this is bad style, but here it is acceptable 
	SetAxis/A/R  /w = linePanel#guiDisplay bottom
	ModifyGraph  /w = linePanel#guiDisplay mirror(left)=2
	ModifyGraph  /w = linePanel#guiDisplay mirror(bottom)=2

	Cursor /H=1 /F /P /S=0 /L=1 /w=linePanel#guiDisplay A LP_DispWave, 0.5,0.5
end

//cancel-button, leave without saving
function LeaveLine(ctrlName):ButtonControl
	string ctrlName
	SVAR globalvalue = LP_nameWaveRed
	string waveTag = globalvalue + "_Bl"
	wave resultWave = $waveTag
	DoWindow /k linePanel
	killwaves /z LP_DispWave, LP_DispWave2, XLP_DispWave, BaseLine, resultWave, XCoords, YCoords
	killstrings /Z root:LP_nameWaveRed
	killstrings /Z root:LP_workingFolder
end


function exitLineWindow(ctrlName):ButtonControl
	string ctrlName
	 
	DoWindow  /k linePanel
	killwaves /z LP_DispWave, LP_DispWave2, XLP_DispWave, BaseLine, xCoords, yCoords, xVals, yVals, output
	killstrings /Z LP_nameWaveRed
	killstrings /Z root:LP_workingFolder
end


function printMe(ctrlName):ButtonControl
	string ctrlName
	string name = "linePanel#guiDisplay"    //hard-coded to avoid more global variables
	 //get the name of the selected wave
	 SVAR RawWaveName = LP_nameWaveRed
	 Button cancBut disable=0
	 GeneratePolygon(RawWaveName)
end


function AddCoord(ctrlName):ButtonControl
	string ctrlName
	string name = "linePanel#guiDisplay" 
	variable indexA, posA, heightA
	//make a reference to the coordinate waves - those have to be present in the current data folder
	wave xVals = XCoords
	wave yVals = YCoords
	SVAR RawWaveName = LP_nameWaveRed
	variable length = numpnts(XCoords)
	
	redimension /d /n=(length + 1)  xVals
	redimension /d /n=(length + 1) yVals
	//now fill in the cursor coordinates into xVals and yVals
	// those values are later on used to generate the polygon background
	indexA = pcsr(A,name)   
	posA = hcsr(A, name)
	heightA= vcsr(A,name)
	
	xVals[length + 1] = posA
	yVals[length + 1] = heightA
	
	if (numpnts(xVals) ==1)
		AppendToGraph /w=linePanel#guiDisplay YCoords vs XCoords
		ModifyGraph /w=linePanel#guiDisplay mode(YCoords)=3, marker(YCoords)=19    //It is necessary to name the window, otherwise: possibly(!) error
	elseif (numpnts(xVals) >= 2)
		GeneratePolygon(RawWaveName)
	endif
end


function DelCoord(ctrlName):ButtonControl
	string ctrlName
	//make a reference to the coordinate waves
	wave xVals = XCoords
	wave yVals = YCoords
	SVAR RawWaveName = LP_nameWaveRed
	
	variable length = numpnts(XCoords)
	
	if ( length > 0)
		redimension /n=(length - 1)  xVals
		redimension /n=(length - 1) yVals
	endif
	
	if (numpnts(xVals) >= 2)
		GeneratePolygon(RawWaveName)
	endif
end



//Calculate a polygon background for XPS data
//.................................................................

function GeneratePolygon(waveTag)
	string waveTag  //this is the name of the raw wave
	wave rawWave = $waveTag
	variable i, j
	string newWaveTag    //name of the final output wave
	
	wave xRawVals = XCoords
	wave yRawVals = YCoords
	duplicate /o xRawVals, xVals
	duplicate /o yRawVals, yVals
	wave xVals = xVals
	wave yVals = yVals
	//now sort the entries accordingly 
	Sort xVals,xVals,yVals   
	 
	variable numXYpoints = numpnts(XCoords)
	variable slope, offset
	
	//make the wave to keep the polygon and then the outputwave
	duplicate /o $waveTag output
	wave output = output
	
	//acquire information on the x-values of the raw wave   //right is always the smallest value
	variable left = max(leftx(rawWave),rightx(rawWave))   
	variable right = min(leftx(rawWave),rightx(rawWave))
	variable xStep = abs(left - right)/numpnts(rawWave)
	//print left, right,xStep
	
	//find out whether if the values in the wave are ascending or descending
	variable currentPos 
	
	variable trend              //1 is descending , 0 is ascending
	if ( leftx(rawWave) > rightx(rawWave))          // as in a usual .itx
		currentPos = left
		trend = 1
	else
		currentPos = right            //in an ascii
		trend = 0
	endif
	
	//now go through the output wave and assign new values
	for ( i = 0; i < numpnts(rawWave); i += 1)
		if (trend == 0)          //the subsequent values are ascending
			currentPos += xStep   
		else
			currentPos -= xStep
		endif
		
		for ( j = 1;  j < numXYPoints; j += 1)   //the first polygon point is used only to create the straight line between 1 and 2
			if (currentPos < xVals[j])
				slope = ( yVals[ j ] - yVals[ j - 1 ] ) / ( xVals[ j ] - xVals[ j - 1 ] )
				offset = yVals[ j ] - slope*xVals[ j ]
				output[i] = offset + slope * currentPos
				break  //leave the j loop on the first success
			endif 
			if (currentPos > xVals[numXYPoints-1])
				slope = ( yVals[numXYPoints-1 ] - yVals[ numXYPoints-2 ] ) / ( xVals[ numXYPoints-1] - xVals[ numXYPoints-2 ] )
				offset = yVals[ numXYPoints-1 ] - slope*xVals[ numXYPoints-1 ]
				output[i] = offset + slope * currentPos
				
			endif
		endfor
	endfor
	
	//now display the whole stuff .............................................
	//.....................................................................................
	
	RemoveFromGraph /Z /w = linePanel#guiDisplay2 $"#0"
	RemoveFromGraph /Z /w =linePanel#guiDisplay output
	
	newWaveTag = waveTag + "_Bl"
	duplicate /o /d rawWave, $newWaveTag
	wave finalOutput = $newWaveTag
	
	RemoveFromGraph /Z /w = linePanel#guiDisplay $"#2"
	
	finalOutput -= output
	AppendToGraph /w =linePanel#guiDisplay output
	AppendToGraph /w =linePanel#guiDisplay2 finalOutput 
	SetAxis/A/R  /w = linePanel#guiDisplay2 bottom
	ModifyGraph /w= linePanel#guiDisplay2 zero(left)=2
	ModifyGraph  /w = linePanel#guiDisplay2 mirror(left)=2
	ModifyGraph  /w = linePanel#guiDisplay2 mirror(bottom)=2
	ModifyGraph  /w = linePanel#guiDisplay2 rgb($newWaveTag)=(0,26112,39168)
	
end
