#pragma rtGlobals=1		// Use modern global access method.
#pragma hide=1

//////////////////////////////////////////////////////////////////////////////////
//Licence
//////////////////////////////////////////////////////////////////////////////////
//
//	If you use the 'Fit Assistant' to analyze data, please read and cite the following paper:
//	Surface and Interface Analysis, (2014), 46, 505-511
//	This publication contains more details on the fit function used by XPST.
//
//	XPST is a front-end to the IgorPro fitting engine. Although every possible effort was made 
//	to prevent serious bugs and wrong results, the author can not give any warranty for XPST as 
//	large parts actually depend on the underlying IgorPro code. 
//	Author: Dr. Martin Schmid
//
//////////////////////////////////////////////////////////////////////////////////

menu "XPST"
	submenu "Background subtraction"
		"Shirley background \ Straight Line", LaunchShirlPanel()
	end
end

macro LaunchShirlPanel()
	String /G root:SP_workingFolder
	DoWindow ShirlPanel     //this writes a 1 into V_Flag, if the panel is already open
	if (V_Flag)    //if it already exists
		DoWindow /F ShirlPanel    //bring  it to front
		return -1   //and leave without anything else
	endif
	root:SP_workingFolder = GetDataFolder(1)
	ShirlPanel()
	
end

//define the gui and the global variables

Window ShirlPanel() : Panel
	GetWindow kwFrameInner, wsize   //get the geometry of the Igor main window
	variable startX, startY, xWidth, yWidth
	
	xWidth = 895
	yWidth = 458
	startx = (V_left +V_right)/7
	starty = 10
	
	
	initializeShirl()
	PauseUpdate; Silent 1		// building window...
	NewPanel  /K=2 /W=(startX, startY, startX  + xWidth, startY + yWidth)
	DoWindow /T ShirlPanel, "Shirley Background \ Straight Line"
	GroupBox PopUpGroup,pos={10,10},size={270,120},title=""
	DrawText 30,34,"Please select a wave"
	
	PopupMenu WavePop,pos={0,45},size={266,24},bodyWidth=240,proc=initShirlPopUp
	PopupMenu WavePop,mode=1,popvalue=" ",value= #"WaveList(\"!*Disp*\",\";\",\"\")"
	
	Button okBut,pos={292,10},size={140,26},disable=2,proc=dumpOut,title="Calculate Shirley"
	Button linearBut, pos={292,40},size={140,26},disable=2,  proc=subtractLine, title="Straight Line"
	Button CancBut, pos={292,70},size={140,26},disable=2,proc=LeaveShirl, title="Undo and Exit"
	Button ExitBut,pos={292,100},size={140,26},proc=exitShirlWindow,title="Accept and Exit"
		
	CheckBox ShirlCheck,pos={25,80},size={109,20},proc=IterateCheck,title="Use iterated Shirley"
	CheckBox ShirlCheck,value= 1
	
	CheckBox FreeCheck,pos={25,103},size={109,20},proc=FreeCheck,title="Bind Cursor to wave"
	CheckBox FreeCheck,value= 1, disable=2
	
	GroupBox Frame2,pos={445, 7},size={445, 445},title=""
	Display/W=( 450, 12, 885, 447 )/HOST=#
	RenameWindow #,guiDisplay
	
	SetActiveSubwindow ShirlPanel
	
	GroupBox Frame1,pos={10, 134},size={426, 315},title=""
	Display/W=(16,140,432,440)/HOST=#
	
	RenameWindow #,guiDisplay2
	
	SetActiveSubwindow ShirlPanel
	
EndMacro

function LeaveShirl(ctrlName):ButtonControl
	string ctrlName
	//SVAR value = SP_nameWaveRed
	//string notIteratedWaveTag = value + ".S"
	//string iteratedWaveTag = value + ".iS"
	//string lineWaveTag = value + ".l"
	//wave shirleyWave = $notIteratedWaveTag
	//wave lineWave = $lineWaveTag
	//wave shirleyIteratedWave = $iteratedWaveTag
	
	DoWindow  /k ShirlPanel
	killwaves /z SP_DispWave, SP_DispWave2, XSP_DispWave, shirleyBack, shirleyWave, LineSubtractWave, lineWave, shirleyIteratedWave
	killwaves /z SP_workWave, SP_ShirleyWave
	killstrings /Z SP_nameWaveRed, root:SP_workingFolder
	killvariables /Z V_left, V_right, V_top, V_bottom, SP_FreeCheck
end

function initializeShirl()
	SVAR name = root:SP_workingFolder
	SetDataFolder name
	Variable /G SP_IterateCheck = 1    //0 or 1, wether the "iterate Shirley"-CheckBox is activated or not
	Variable /G SP_FreeCheck = 1    //0 or 1, wether the "free cursor"-CheckBox is activated or not
	String /G SP_nameWaveRed = StringFromList(0,WaveList("!*Disp*",";",""))
end

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function to react to the Check-Box "ShirlCheck"
///

function IterateCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	NVAR value = SP_IterateCheck
	value = checked
end


function FreeCheck(ctrlName,checked): CheckBoxControl
	string ctrlName
	variable checked
	NVAR value = SP_FreeCheck
	value = checked
	if (checked == 0)
		Cursor /F /P  / w=ShirlPanel#guiDisplay A  $"SP_DispWave" ,0.2, 0.5
		Cursor /F /P  / w=ShirlPanel#guiDisplay B  $"SP_DispWave" ,0.9, 0.5
	elseif (checked == 1 )
		print "wave evaluation left/right  ", leftx($"SP_DispWave"), " / ", rightx($"SP_DispWave")
		Cursor  /w=ShirlPanel#guiDisplay A  $"SP_DispWave", leftx($"SP_DispWave") 
		Cursor /w = ShirlPanel#guiDisplay B $"SP_DispWave", rightx($"SP_DispWave") 
	endif
end


function initShirlPopUp(ctrlName,popNum,popStr): PopupMenuControl
//as soon as a wave is selected, it is displayed and cursors appear
	string ctrlName
	variable popNum
	String popStr
	//local reference to a global variable
	SVAR value = SP_nameWaveRed
	value = popStr                              //write the selection to the global string
	variable highBindPos, lowBindPos
	variable switchVar
	
	Button okBut disable=0
	Button linearBut disable=0
	Button CancBut disable=2
	wave WorkWave = $value
	CheckBox FreeCheck, disable = 0, value = 1
	duplicate /o /d WorkWave, SP_DispWave
	WaveStats /Q WorkWave
	
	RemoveFromGraph /Z /W=ShirlPanel#guiDisplay /Z $"#0" $"#1" $"#2" $"#3" $"#4"   //more traces..to be on the safe side
	RemoveFromGraph /Z /w = ShirlPanel#guiDisplay2 $"#0", $"#1"
	AppendToGraph /W=ShirlPanel#guiDisplay SP_DispWave
	ModifyGraph /W=ShirlPanel#guiDisplay rgb(SP_DispWave)=(0,0,0) 
	ModifyGraph /W=ShirlPanel#guiDisplay minor(bottom) = 1
	//the name of the panel is hard-coded, usually this is bad style, but here it is acceptable 
	//print "wave evaluation left/right  ", leftx($"SP_DispWave"), " / ", rightx($"SP_DispWave")
	highBindPos = leftx($"SP_DispWave")
	lowBindPos = rightx($"SP_DispWave")
	if ( highBindPos < lowBindPos)
	 	switchVar = highBindPos
	 	highBindPos = lowBindPos
	 	lowBindPos = switchVar
	 endif
	Cursor  /w=ShirlPanel#guiDisplay A  SP_DispWave, highBindPos
	Cursor /w = ShirlPanel#guiDisplay B SP_DispWave, lowBindPos 
	
	//Cursor /F /P  / w=ShirlPanel#guiDisplay A  SP_DispWave,0.2, 0.5
	//Cursor /F /P  / w=ShirlPanel#guiDisplay B  SP_DispWave,0.9, 0.5
	SetAxis /w=ShirlPanel#guiDisplay left 0.9*V_min, 1.15*V_max
	TextBox /w=ShirlPanel#guiDisplay /N=legendText /K        //delete the textbox if it is there
	TextBox /w=ShirlPanel#guiDisplay /N=legendText /F=0 /A = LT "Use  cursors to set the background\rX position is irrelevant for Shirley"
	SetAxis/A/R  /w = ShirlPanel#guiDisplay bottom
	ModifyGraph  /w = ShirlPanel#guiDisplay mirror(left)=2
	ModifyGraph  /w = ShirlPanel#guiDisplay mirror(bottom)=2
	
	//SP_DispWave is deletetd when the panel is closed	 
end

function exitShirlWindow(ctrlName):ButtonControl
	string ctrlName
	 
	DoWindow  /k ShirlPanel
	killwaves /z SP_DispWave, SP_DispWave2, XSP_DispWave, shirleyBack
	killwaves /z SP_ShirleyWave, SP_workWave
	killstrings /Z SP_nameWaveRed, root:SP_workingFolder
	killvariables /Z SP_IterateCheck, SP_FreeCheck
	killvariables /Z V_left, V_right, V_top, V_bottom
end


function dumpOut(ctrlName):ButtonControl
	string ctrlName
	string name = "ShirlPanel#guiDisplay"    //hard-coded to avoid more global variables
	 //get the name of the selected wave
	 SVAR workWaveName = SP_nameWaveRed
	 variable switchVar
	 
	//read out the curosr position
	 variable posA = hcsr(A, name)
	 variable heightA= vcsr(A,name)
	 print " "
	 print "Position of A (round)", posA,heightA
	 
	 variable posB = hcsr(B,name)
	 variable heightB=vcsr(B,name)
	 print "Position of B (rect)", posB, heightB
	 if ( posA < posB)
		switchVar = heightA
		heightA = heightB 
	 	heightB = switchVar
		switchVar = posA
		posA = posB
	 	posB = switchVar
	 endif
	 Button CancBut disable=0
	 //DoShirley(value,posA,heightA, posB, heightB)	
	IterateShirl(workWaveName,posA,heightA, posB, heightB)
end
	
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Iterated and not-iterated Shirley background
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function IterateShirl(waveTag, startx, starty, endx, endy)
	string waveTag
	variable startx, starty
	variable endx, endy
	
	//check wether the wave has the regular XPS order = "from high energy to low energy" 
	//or it is reverse "from low energy to high energy"
	variable highBindPos = leftx($"SP_DispWave")
	variable lowBindPos = rightx($"SP_DispWave")
	
	variable orderIndicator = 0 // 0 means everything is all right, i.e. from "high to low"
	
	if ( lowBindPos >= highBindPos)
		orderIndicator = 1
	endif
	
	wave rawWave = $waveTag
	duplicate /o rawWave SP_workWave
	duplicate /o rawWave SP_ShirleyWave
	
	variable wholeArea 
	variable tempSum
	
	//integer variables
	NVAR checked = SP_IterateCheck
	variable i,j,k
	variable Iterations = 10          				 //default: use iterated shirley
	variable wavelength = dimSize(rawWave,0)
	
	//check, wether to use the iterated shirley or not
	if (checked == 0)
		Iterations =1
		//print "Number of iterations:", Iterations
	endif
	
	SP_ShirleyWave = endy      //Initialize the first shirley-wave as the offset, before starting the iteration
	for (k = 0; k < Iterations; k += 1)
		SP_workWave = rawWave - SP_ShirleyWave
		//take only positive areas into account, otherwise everything gets numerically uncertain
		wholeArea=0
		for (j = 0; j< wavelength; j+=1)      
			wholeArea += abs(SP_workWave[j])
		endfor
		
		//the following step depends, if the wave is in regular order or reverese		
		if ( orderIndicator == 0)                  //if this is the case, just go through the wave
			for (i = 0; i<wavelength; i+=1)
				tempSum=0
				for (j = i; j < wavelength; j += 1)
					tempSum += abs(SP_workWave[j])
				endfor
				SP_ShirleyWave[i] = tempSum
			endfor
		else 
			for (i = wavelength-1; i > 0; i -= 1)     //the wave is reverse, start from bottom
				tempSum=0
				for (j = i; j > 0; j -= 1)
					tempSum += abs(SP_workWave[j])
				endfor
				SP_ShirleyWave[i] = tempSum
			endfor
			
		endif
		SP_ShirleyWave = (starty-endy)*SP_ShirleyWave / wholeArea
	endfor
	
	SP_workWave = rawWave - SP_ShirleyWave - endy
	
	//duplicate the resulting wave and name the clone, according to the raw data
	if ( checked != 0) 
		duplicate /o SP_workWave $waveTag+"_iS"
	else
		duplicate /o SP_workWave $waveTag+"_S"
	endif 
	
	//add offset to the SP_ShirleyWave, to display it properly together with the raw-data
	SP_ShirleyWave += endy
	WaveStats /Q SP_workWave
	//plot
	RemoveFromGraph /Z /w = ShirlPanel#guiDisplay $"#1"  //If there is something, remove it
	RemoveFromGraph /Z /w = ShirlPanel#guiDisplay2 $"#0", $"#1"  //If there is something, remove it
	AppendToGraph /w =ShirlPanel#guiDisplay2 SP_workWave
	AppendToGraph /w =ShirlPanel#guiDisplay SP_ShirleyWave
	
	SetAxis/A/R  /w = ShirlPanel#guiDisplay2 bottom
	ModifyGraph /w=ShirlPanel#guiDisplay2 zero(left)=2
	ModifyGraph  /w = ShirlPanel#guiDisplay2 mirror(left)=2
	ModifyGraph  /w = ShirlPanel#guiDisplay2 mirror(bottom)=2
	ModifyGraph  /w = ShirlPanel#guiDisplay2 rgb(SP_workWave)=(0,26112,39168)
	ModifyGraph  /w = ShirlPanel#guiDisplay2 minor(bottom)=1
	SetAxis /w=ShirlPanel#guiDisplay2 left -0.1*V_max, 1.15*V_max
end


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// subtract a straight line
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function subtractLine(ctrlName):ButtonControl
	string ctrlName
	string name = "ShirlPanel#guiDisplay"    //hard-coded to avoid more global variables
	variable slope
	 variable switchVar
	//get the name of the selected wave
	SVAR rawWaveName = SP_nameWaveRed        //get the name of the pre-existing wave
	wave WorkingWave = $rawWaveName                //and make a local reference to it
	 
	string waveTag = rawWaveName + "_l"                //get the name
	duplicate /o WorkingWave $waveTag			//create a (global) wave with this name
	wave ResultWave = $waveTag                            //and make a local reference to it
	
	duplicate /o WorkingWave SP_DispWave2          //create a wave to hold the straight line
	
	//read out the curosr position
	 variable posA = hcsr(A, name)
	 variable heightA= vcsr(A,name)
	 
	 variable posB = hcsr(B,name)
	 variable heightB=vcsr(B,name)
	 
	 if ( posA < posB)
		switchVar = heightA
		heightA = heightB 
	 	heightB = switchVar
	 	switchVar = posA
	 	posA = posB
	 	posB = switchVar
	 endif
	 
	 //calculate the straight line
	slope = (heightA-heightB)/(posA-posB)        //posA>posB
	ResultWave = ResultWave - (slope*(x-posB) + heightB)
	SP_DispWave2 = slope*(x-posB) + heightB

	WaveStats /Q ResultWave
	//and display it 
	RemoveFromGraph /Z /w = ShirlPanel#guiDisplay $"#1"  //If there is something, remove it
	RemoveFromGraph /Z /w = ShirlPanel#guiDisplay2 $"#0", $"#1"  //If there is something, remove it
	AppendToGraph /w =ShirlPanel#guiDisplay2 ResultWave
	AppendToGraph /w=ShirlPanel#guiDisplay SP_DispWave2
	SetAxis/A/R  /w = ShirlPanel#guiDisplay2 bottom
	ModifyGraph /w=ShirlPanel#guiDisplay2 zero(left)=2
	ModifyGraph  /w = ShirlPanel#guiDisplay2 mirror(left)=2
	ModifyGraph  /w = ShirlPanel#guiDisplay2 mirror(bottom)=2
	ModifyGraph  /w = ShirlPanel#guiDisplay2 rgb($waveTag)=(0,26112,39168)

	
	ModifyGraph  /w = ShirlPanel#guiDisplay2 minor(bottom)=1
	SetAxis /w=ShirlPanel#guiDisplay2 left -0.1*V_max, 1.15*V_max
	Button CancBut disable=0 
end
